@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,24 +13,24 @@
 #
 # END HEADER
 
-# -*- coding: utf-8 -*-
-
-from __future__ import absolute_import, division, print_function
-
 import datetime
 import os
 import sys
 
+import sphinx_rtd_theme
+
 sys.path.append(os.path.join(os.path.dirname(__file__), "..", "src"))
 
 
 autodoc_member_order = "bysource"
+autodoc_typehints = "none"
 
 extensions = [
     "sphinx.ext.autodoc",
     "sphinx.ext.extlinks",
     "sphinx.ext.viewcode",
     "sphinx.ext.intersphinx",
+    "hoverxref.extension",
 ]
 
 templates_path = ["_templates"]
@@ -43,9 +41,9 @@ source_suffix = ".rst"
 master_doc = "index"
 
 # General information about the project.
-project = u"Hypothesis"
-copyright = u"2013-%s, David R. MacIver" % datetime.datetime.utcnow().year
-author = u"David R. MacIver"
+project = "Hypothesis"
+copyright = "2013-%s, David R. MacIver" % datetime.datetime.utcnow().year
+author = "David R. MacIver"
 
 _d = {}
 with open(
@@ -63,9 +61,13 @@ pygments_style = "sphinx"
 
 todo_include_todos = False
 
+# See https://sphinx-hoverxref.readthedocs.io/en/latest/configuration.html
+hoverxref_auto_ref = True
+hoverxref_domains = ["py"]
+
 intersphinx_mapping = {
     "python": ("https://docs.python.org/3/", None),
-    "numpy": ("https://docs.scipy.org/doc/numpy/", None),
+    "numpy": ("https://numpy.org/doc/stable/", None),
     "pandas": ("https://pandas.pydata.org/pandas-docs/stable/", None),
     "pytest": ("https://docs.pytest.org/en/stable/", None),
     "django": ("https://django.readthedocs.io/en/stable/", None),
@@ -86,16 +88,15 @@ extlinks = {
     "pull": (_repo + "pull/%s", "pull request #"),
     "pypi": ("https://pypi.org/project/%s", ""),
     "bpo": ("https://bugs.python.org/issue%s", "bpo-"),
+    "np-ref": ("https://numpy.org/doc/stable/reference/%s", ""),
+    "wikipedia": ("https://en.wikipedia.org/wiki/%s", ""),
 }
 
 # -- Options for HTML output ----------------------------------------------
 
-if os.environ.get("READTHEDOCS", None) != "True":
-    # only import and set the theme if we're building docs locally
-    import sphinx_rtd_theme
+html_theme = "sphinx_rtd_theme"
 
-    html_theme = "sphinx_rtd_theme"
-    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
+html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
 
 html_static_path = ["_static"]
 
@@ -108,22 +109,16 @@ html_favicon = "../../brand/favicon.ico"
 latex_elements = {}
 
 latex_documents = [
-    (
-        master_doc,
-        "Hypothesis.tex",
-        u"Hypothesis Documentation",
-        u"David R. MacIver",
-        "manual",
-    )
+    (master_doc, "Hypothesis.tex", "Hypothesis Documentation", author, "manual")
 ]
 
-man_pages = [(master_doc, "hypothesis", u"Hypothesis Documentation", [author], 1)]
+man_pages = [(master_doc, "hypothesis", "Hypothesis Documentation", [author], 1)]
 
 texinfo_documents = [
     (
         master_doc,
         "Hypothesis",
-        u"Hypothesis Documentation",
+        "Hypothesis Documentation",
         author,
         "Hypothesis",
         "Advanced property-based testing for Python.",
