@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,8 +13,6 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import math
 import sys
 from sys import float_info
@@ -46,35 +42,31 @@ def test_cathetus_huge_no_overflow():
     h = sys.float_info.max
     a = h / math.sqrt(2)
     b = cathetus(h, a)
-    assert not (
-        math.isinf(b) or math.isnan(b)
-    ), "expecting finite cathetus(%g, %g), got %g" % (h, a, b)
+    assert math.isfinite(b), "expecting finite cathetus(%g, %g), got %g" % (h, a, b)
 
 
 def test_cathetus_large_no_overflow():
     h = sys.float_info.max / 3
     a = h / math.sqrt(2)
     b = cathetus(h, a)
-    assert not (
-        math.isinf(b) or math.isnan(b)
-    ), "expecting finite cathetus(%g, %g), got %g" % (h, a, b)
+    assert math.isfinite(b), "expecting finite cathetus(%g, %g), got %g" % (h, a, b)
 
 
 @pytest.mark.parametrize(
     "h,a",
     [
         # NaN hypot
-        (float(u"nan"), 3),
-        (float(u"nan"), 0),
-        (float(u"nan"), float(u"inf")),
-        (float(u"nan"), float(u"nan")),
+        (math.nan, 3),
+        (math.nan, 0),
+        (math.nan, math.inf),
+        (math.nan, math.nan),
         # Infeasible
         (2, 3),
         (2, -3),
-        (2, float(u"inf")),
-        (2, float(u"nan")),
+        (2, math.inf),
+        (2, math.nan),
         # Surprisingly consistent with c99 hypot()
-        (float(u"inf"), float(u"inf")),
+        (math.inf, math.inf),
     ],
 )
 def test_cathetus_nan(h, a):
@@ -82,13 +74,7 @@ def test_cathetus_nan(h, a):
 
 
 @pytest.mark.parametrize(
-    "h,a",
-    [
-        (float(u"inf"), 3),
-        (float(u"inf"), -3),
-        (float(u"inf"), 0),
-        (float(u"inf"), float(u"nan")),
-    ],
+    "h,a", [(math.inf, 3), (math.inf, -3), (math.inf, 0), (math.inf, math.nan)],
 )
 def test_cathetus_infinite(h, a):
     assert math.isinf(cathetus(h, a))
