@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,14 +13,10 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import assume, given
+from hypothesis import assume, given, strategies as st
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import hrange
 from tests.common.debug import minimal
 from tests.common.utils import flaky
 
@@ -30,7 +24,7 @@ from tests.common.utils import flaky
 @st.composite
 def badly_draw_lists(draw, m=0):
     length = draw(st.integers(m, m + 10))
-    return [draw(st.integers()) for _ in hrange(length)]
+    return [draw(st.integers()) for _ in range(length)]
 
 
 def test_simplify_draws():
