@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,12 +13,9 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import given
+from hypothesis import given, strategies as st
 
 
 def test_includes_non_default_args_in_repr():
@@ -28,6 +23,11 @@ def test_includes_non_default_args_in_repr():
     assert repr(st.integers(min_value=1)) == "integers(min_value=1)"
 
 
+def test_sampled_repr_leaves_range_as_range():
+    huge = 10 ** 100
+    assert repr(st.sampled_from(range(huge))) == "sampled_from(range(0, %s))" % (huge,)
+
+
 def hi(there, stuff):
     return there
 
@@ -40,10 +40,11 @@ def test_supports_positional_and_keyword_args_in_builds():
 
 
 def test_preserves_sequence_type_of_argument():
-    assert repr(st.sampled_from([0])) == "sampled_from([0])"
+    assert repr(st.sampled_from([0, 1])) == "sampled_from([0, 1])"
+    assert repr(st.sampled_from((0, 1))) == "sampled_from((0, 1))"
 
 
-class IHaveABadRepr(object):
+class IHaveABadRepr:
     def __repr__(self):
         raise ValueError("Oh no!")
 
