@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,14 +13,13 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import datetime as dt
+import sys
 
 import pytest
 
 from hypothesis import given, settings
-from hypothesis.internal.compat import hrange
+from hypothesis.internal.compat import PYPY
 from hypothesis.strategies import dates, datetimes, timedeltas, times
 from tests.common.debug import find_any, minimal
 
@@ -94,7 +91,7 @@ def test_can_find_before_the_year_2000():
     assert minimal(dates(), lambda x: x.year < 2000).year == 1999
 
 
-@pytest.mark.parametrize("month", hrange(1, 13))
+@pytest.mark.parametrize("month", range(1, 13))
 def test_can_find_each_month(month):
     find_any(dates(), lambda x: x.month == month, settings(max_examples=10 ** 6))
 
@@ -140,3 +137,15 @@ def test_can_generate_naive_time():
 @given(times())
 def test_naive_times_are_naive(dt):
     assert dt.tzinfo is None
+
+
+# The fold attribute was added in Python 3.6.  It's less clear why this also fails
+# on pypy3.6, but it seems to canonicalise fold to 0 for non-ambiguous times...
+@pytest.mark.skipif(PYPY or sys.version_info[:2] < (3, 6), reason="see comment")
+def test_can_generate_datetime_with_fold_1():
+    find_any(datetimes(), lambda d: d.fold)
+
+
+@pytest.mark.skipif(PYPY or sys.version_info[:2] < (3, 6), reason="see comment")
+def test_can_generate_time_with_fold_1():
+    find_any(times(), lambda d: d.fold)
