@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,34 +13,41 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
+import time
 from unittest import TestCase
 
 import pytest
 
-from hypothesis import Phase, Verbosity, example, given, note, reporting, settings
-from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import integer_types, print_unicode
-from hypothesis.strategies import integers, text
-from tests.common.utils import capture_out
+from hypothesis import (
+    Phase,
+    Verbosity,
+    assume,
+    example,
+    given,
+    note,
+    reporting,
+    settings,
+)
+from hypothesis.errors import DeadlineExceeded, InvalidArgument, MultipleFailures
+from hypothesis.strategies import floats, integers, nothing, text
+from tests.common.utils import assert_falsifying_output, capture_out
 
 
 class TestInstanceMethods(TestCase):
     @given(integers())
     @example(1)
     def test_hi_1(self, x):
-        assert isinstance(x, integer_types)
+        assert isinstance(x, int)
 
     @given(integers())
     @example(x=1)
     def test_hi_2(self, x):
-        assert isinstance(x, integer_types)
+        assert isinstance(x, int)
 
     @given(x=integers())
     @example(x=1)
     def test_hi_3(self, x):
-        assert isinstance(x, integer_types)
+        assert isinstance(x, int)
 
 
 def test_kwarg_example_on_testcase():
@@ -50,9 +55,9 @@ def test_kwarg_example_on_testcase():
         @given(integers())
         @example(x=1)
         def test_hi(self, x):
-            assert isinstance(x, integer_types)
+            assert isinstance(x, int)
 
-    Stuff(u"test_hi").test_hi()
+    Stuff("test_hi").test_hi()
 
 
 def test_errors_when_run_with_not_enough_args():
@@ -76,7 +81,7 @@ def test_errors_when_run_with_not_enough_kwargs():
 
 
 def test_can_use_examples_after_given():
-    long_str = u"This is a very long string that you've no chance of hitting"
+    long_str = "This is a very long string that you've no chance of hitting"
 
     @example(long_str)
     @given(text())
@@ -88,7 +93,7 @@ def test_can_use_examples_after_given():
 
 
 def test_can_use_examples_before_given():
-    long_str = u"This is a very long string that you've no chance of hitting"
+    long_str = "This is a very long string that you've no chance of hitting"
 
     @given(text())
     @example(long_str)
@@ -100,8 +105,8 @@ def test_can_use_examples_before_given():
 
 
 def test_can_use_examples_around_given():
-    long_str = u"This is a very long string that you've no chance of hitting"
-    short_str = u"Still no chance"
+    long_str = "This is a very long string that you've no chance of hitting"
+    short_str = "Still no chance"
 
     seen = []
 
@@ -115,7 +120,7 @@ def test_can_use_examples_around_given():
     assert set(seen[:2]) == {long_str, short_str}
 
 
-@pytest.mark.parametrize((u"x", u"y"), [(1, False), (2, True)])
+@pytest.mark.parametrize(("x", "y"), [(1, False), (2, True)])
 @example(z=10)
 @given(z=integers())
 def test_is_a_thing(x, y, z):
@@ -143,7 +148,7 @@ def test_does_not_print_on_explicit_examples_if_no_failure():
             with capture_out() as out:
                 test_positive()
     out = out.getvalue()
-    assert u"Falsifying example: test_positive(1)" not in out
+    assert "Falsifying example: test_positive(1)" not in out
 
 
 def test_prints_output_for_explicit_examples():
@@ -152,12 +157,7 @@ def test_prints_output_for_explicit_examples():
     def test_positive(x):
         assert x > 0
 
-    with reporting.with_reporter(reporting.default):
-        with pytest.raises(AssertionError):
-            with capture_out() as out:
-                test_positive()
-    out = out.getvalue()
-    assert u"Falsifying example: test_positive(x=-1)" in out
+    assert_falsifying_output(test_positive, x=-1)
 
 
 def test_prints_verbose_output_for_explicit_examples():
@@ -167,11 +167,9 @@ def test_prints_verbose_output_for_explicit_examples():
     def test_always_passes(x):
         pass
 
-    with reporting.with_reporter(reporting.default):
-        with capture_out() as out:
-            test_always_passes()
-    out = out.getvalue()
-    assert u"Trying example: test_always_passes(x='NOT AN INTEGER')" in out
+    assert_falsifying_output(
+        test_always_passes, x="NOT AN INTEGER", example_type="Trying",
+    )
 
 
 def test_captures_original_repr_of_example():
@@ -181,12 +179,9 @@ def test_captures_original_repr_of_example():
         x.append(1)
         assert not x
 
-    with reporting.with_reporter(reporting.default):
-        with pytest.raises(AssertionError):
-            with capture_out() as out:
-                test_mutation()
-    out = out.getvalue()
-    assert u"Falsifying example: test_mutation(x=[])" in out
+    assert_falsifying_output(
+        test_mutation, x=[],
+    )
 
 
 def test_examples_are_tried_in_order():
@@ -196,13 +191,13 @@ def test_examples_are_tried_in_order():
     @settings(phases=[Phase.explicit])
     @example(x=3)
     def test(x):
-        print_unicode(u"x -> %d" % (x,))
+        print("x -> %d" % (x,))
 
     with capture_out() as out:
         with reporting.with_reporter(reporting.default):
             test()
     ls = out.getvalue().splitlines()
-    assert ls == [u"x -> 1", u"x -> 2", u"x -> 3"]
+    assert ls == ["x -> 1", "x -> 2", "x -> 3"]
 
 
 def test_prints_note_in_failing_example():
@@ -218,7 +213,7 @@ def test_prints_note_in_failing_example():
             with pytest.raises(AssertionError):
                 test()
     v = out.getvalue()
-    print_unicode(v)
+    print(v)
     assert "x -> 43" in v
     assert "x -> 42" not in v
 
@@ -231,3 +226,37 @@ def test_must_agree_with_number_of_arguments():
 
     with pytest.raises(InvalidArgument):
         test()
+
+
+def test_runs_deadline_for_examples():
+    @example(10)
+    @settings(phases=[Phase.explicit])
+    @given(nothing())
+    def test(x):
+        time.sleep(10)
+
+    with pytest.raises(DeadlineExceeded):
+        test()
+
+
+@given(value=floats(0, 1))
+@example(value=0.56789)
+@pytest.mark.parametrize("threshold", [0.5, 1])
+def test_unsatisfied_assumption_during_explicit_example(threshold, value):
+    # Regression test, expected to pass / skip depending on parametrize.
+    # See https://github.com/HypothesisWorks/hypothesis/issues/2125
+    assume(value < threshold)
+
+
+@pytest.mark.parametrize("exc", [MultipleFailures, AssertionError])
+def test_multiple_example_reporting(exc):
+    @example(1)
+    @example(2)
+    @settings(report_multiple_bugs=exc is MultipleFailures, phases=[Phase.explicit])
+    @given(integers())
+    def inner_test_multiple_failing_examples(x):
+        assert x < 2
+        assert x < 1
+
+    with pytest.raises(exc):
+        inner_test_multiple_failing_examples()
