@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,20 +13,17 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import math
 import sys
 import warnings
-from decimal import Decimal
 
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import assume, given
+from hypothesis import assume, given, strategies as st
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import CAN_PACK_HALF_FLOAT, WINDOWS
 from hypothesis.internal.floats import (
+    float_of,
     float_to_int,
     int_to_float,
     is_negative,
@@ -36,7 +31,6 @@ from hypothesis.internal.floats import (
     next_up,
 )
 from tests.common.debug import find_any, minimal
-from tests.common.utils import checks_deprecated_behaviour
 
 try:
     import numpy
@@ -66,7 +60,7 @@ def test_can_generate_both_zeros(sign):
 
 
 @pytest.mark.parametrize(
-    (u"l", u"r"),
+    ("l", "r"),
     [(-1.0, 1.0), (-0.0, 1.0), (-1.0, 0.0), (-sys.float_info.min, sys.float_info.min)],
 )
 @pytest.mark.parametrize("sign", [-1, 1])
@@ -174,28 +168,6 @@ def test_updown_roundtrip(val):
     assert val == next_down(next_up(val))
 
 
-@checks_deprecated_behaviour
-@pytest.mark.parametrize("xhi", [True, False])
-@pytest.mark.parametrize("xlo", [True, False])
-@given(st.data(), st.floats(allow_nan=False, allow_infinity=False).filter(bool))
-def test_floats_in_tiny_interval_within_bounds(xlo, xhi, data, center):
-    assume(not (math.isinf(next_down(center)) or math.isinf(next_up(center))))
-    lo = Decimal.from_float(next_down(center)).next_plus()
-    hi = Decimal.from_float(next_up(center)).next_minus()
-    assert float(lo) < lo < center < hi < float(hi)
-    val = data.draw(st.floats(lo, hi, exclude_min=xlo, exclude_max=xhi))
-    assert lo < val < hi
-
-
-@checks_deprecated_behaviour
-def test_float_free_interval_is_invalid():
-    lo = (2 ** 54) + 1
-    hi = lo + 2
-    assert float(lo) < lo < hi < float(hi), "There are no floats in [lo .. hi]"
-    with pytest.raises(InvalidArgument):
-        st.floats(lo, hi).example()
-
-
 @given(st.floats(width=32, allow_infinity=False))
 def test_float32_can_exclude_infinity(x):
     assert not math.isinf(x)
@@ -210,14 +182,14 @@ def test_float16_can_exclude_infinity(x):
 @pytest.mark.parametrize(
     "kwargs",
     [
-        dict(min_value=10 ** 5, width=16),
-        dict(max_value=10 ** 5, width=16),
-        dict(min_value=10 ** 40, width=32),
-        dict(max_value=10 ** 40, width=32),
-        dict(min_value=10 ** 400, width=64),
-        dict(max_value=10 ** 400, width=64),
-        dict(min_value=10 ** 400),
-        dict(max_value=10 ** 400),
+        {"min_value": 10 ** 5, "width": 16},
+        {"max_value": 10 ** 5, "width": 16},
+        {"min_value": 10 ** 40, "width": 32},
+        {"max_value": 10 ** 40, "width": 32},
+        {"min_value": 10 ** 400, "width": 64},
+        {"max_value": 10 ** 400, "width": 64},
+        {"min_value": 10 ** 400},
+        {"max_value": 10 ** 400},
     ],
 )
 def test_out_of_range(kwargs):
@@ -269,21 +241,21 @@ def test_can_exclude_endpoints(x):
     assert 0 < x < 1
 
 
-@given(st.floats(float("-inf"), -1e307, exclude_min=True))
+@given(st.floats(-math.inf, -1e307, exclude_min=True))
 def test_can_exclude_neg_infinite_endpoint(x):
     assert not math.isinf(x)
 
 
-@given(st.floats(1e307, float("inf"), exclude_max=True))
+@given(st.floats(1e307, math.inf, exclude_max=True))
 def test_can_exclude_pos_infinite_endpoint(x):
     assert not math.isinf(x)
 
 
 def test_exclude_infinite_endpoint_is_invalid():
     with pytest.raises(InvalidArgument):
-        st.floats(min_value=float("inf"), exclude_min=True).validate()
+        st.floats(min_value=math.inf, exclude_min=True).validate()
     with pytest.raises(InvalidArgument):
-        st.floats(max_value=float("-inf"), exclude_max=True).validate()
+        st.floats(max_value=-math.inf, exclude_max=True).validate()
 
 
 @pytest.mark.parametrize("lo,hi", [(True, False), (False, True), (True, True)])
@@ -293,13 +265,37 @@ def test_exclude_entire_interval(lo, hi, bound):
         st.floats(bound, bound, exclude_min=lo, exclude_max=hi).validate()
 
 
-def test_exclude_zero_interval():
+def test_zero_intervals_are_OK():
+    st.floats(0.0, 0.0).validate()
     st.floats(-0.0, 0.0).validate()
-    st.floats(-0.0, 0.0, exclude_min=True).validate()
-    st.floats(-0.0, 0.0, exclude_max=True).validate()
+    st.floats(-0.0, -0.0).validate()
 
 
-@checks_deprecated_behaviour
-def test_inverse_zero_interval_is_deprecated():
-    st.floats(0.0, -0.0).validate()
-    st.floats(-0.0, 0.0, exclude_min=True, exclude_max=True).validate()
+@pytest.mark.parametrize("lo", [0.0, -0.0])
+@pytest.mark.parametrize("hi", [0.0, -0.0])
+@pytest.mark.parametrize("exmin,exmax", [(True, False), (False, True), (True, True)])
+def test_cannot_exclude_endpoint_with_zero_interval(lo, hi, exmin, exmax):
+    with pytest.raises(InvalidArgument):
+        st.floats(lo, hi, exclude_min=exmin, exclude_max=exmax).validate()
+
+
+WIDTHS = (64, 32)
+if numpy or CAN_PACK_HALF_FLOAT:
+    WIDTHS += (16,)
+
+
+@pytest.mark.parametrize("nonfloat", [st.nothing(), st.none()])
+@given(data=st.data(), width=st.sampled_from(WIDTHS))
+def test_fuzzing_floats_bounds(data, width, nonfloat):
+    lo = data.draw(nonfloat | st.floats(allow_nan=False, width=width), label="lo")
+    hi = data.draw(nonfloat | st.floats(allow_nan=False, width=width), label="hi")
+    if lo is not None and hi is not None and lo > hi:
+        lo, hi = hi, lo
+    assume(lo != 0 or hi != 0)
+    value = data.draw(
+        st.floats(min_value=lo, max_value=hi, width=width, allow_nan=False),
+        label="value",
+    )
+    assert value == float_of(value, width=width)
+    assert lo is None or lo <= value
+    assert hi is None or value <= hi
