@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,10 +13,7 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
-import hypothesis.strategies as st
-from hypothesis import given
+from hypothesis import given, strategies as st
 from tests.common.debug import assert_no_examples
 
 
@@ -36,3 +31,9 @@ def test_one_of_filtered(i):
 @given(st.one_of(st.just(100).flatmap(st.integers)))
 def test_one_of_flatmapped(i):
     assert i >= 100
+
+
+def test_one_of_single_strategy_is_noop():
+    s = st.integers()
+    assert st.one_of(s) is s
+    assert st.one_of([s]) is s
