@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,24 +13,21 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import re
 import sys
 import unicodedata
 
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import assume, given, settings
+from hypothesis import HealthCheck, assume, given, settings, strategies as st
+from hypothesis._settings import local_settings
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import PY3, hrange, hunichr
-from hypothesis.searchstrategy.regex import (
+from hypothesis.internal.compat import PYPY
+from hypothesis.strategies._internal.regex import (
     SPACE_CHARS,
     UNICODE_DIGIT_CATEGORIES,
     UNICODE_SPACE_CATEGORIES,
     UNICODE_SPACE_CHARS,
-    UNICODE_WEIRD_NONWORD_CHARS,
     UNICODE_WORD_CATEGORIES,
     base_regex_strategy,
 )
@@ -60,18 +55,12 @@ def is_unicode_space(s):
 
 def is_word(s):
     return all(
-        c == "_"
-        or (
-            (PY3 or c not in UNICODE_WEIRD_NONWORD_CHARS)
-            and unicodedata.category(c) in UNICODE_WORD_CATEGORIES
-        )
-        for c in s
+        c == "_" or unicodedata.category(c) in UNICODE_WORD_CATEGORIES for c in s
     )
 
 
 def ascii_regex(pattern):
-    flags = re.ASCII if PY3 else 0
-    return re.compile(pattern, flags)
+    return re.compile(pattern, re.ASCII)
 
 
 def unicode_regex(pattern):
@@ -81,8 +70,8 @@ def unicode_regex(pattern):
 def _test_matching_pattern(pattern, isvalidchar, is_unicode=False):
     r = unicode_regex(pattern) if is_unicode else ascii_regex(pattern)
 
-    codepoints = hrange(0, sys.maxunicode + 1) if is_unicode else hrange(1, 128)
-    for c in [hunichr(x) for x in codepoints]:
+    codepoints = range(0, sys.maxunicode + 1) if is_unicode else range(1, 128)
+    for c in [chr(x) for x in codepoints]:
         if isvalidchar(c):
             assert r.search(c), (
                 '"%s" supposed to match "%s" (%r, category "%s"), '
@@ -119,78 +108,79 @@ def test_matching(category, predicate, invert, is_unicode):
 @pytest.mark.parametrize(
     "pattern",
     [
-        u".",  # anything
-        u"a",
-        u"abc",
-        u"[a][b][c]",
-        u"[^a][^b][^c]",  # literals
-        u"[a-z0-9_]",
-        u"[^a-z0-9_]",  # range and negative range
-        u"ab?",
-        u"ab*",
-        u"ab+",  # quantifiers
-        u"ab{5}",
-        u"ab{5,10}",
-        u"ab{,10}",
-        u"ab{5,}",  # repeaters
-        u"ab|cd|ef",  # branch
-        u"(foo)+",
-        u"(['\"])[a-z]+\\1",
-        u"(?:[a-z])(['\"])[a-z]+\\1",
-        u"(?P<foo>['\"])[a-z]+(?P=foo)",  # groups
-        u"^abc",  # beginning
-        u"\\d",
-        u"[\\d]",
-        u"[^\\D]",
-        u"\\w",
-        u"[\\w]",
-        u"[^\\W]",
-        u"\\s",
-        u"[\\s]",
-        u"[^\\S]",  # categories
+        ".",  # anything
+        "a",
+        "abc",
+        "[a][b][c]",
+        "[^a][^b][^c]",  # literals
+        "[a-z0-9_]",
+        "[^a-z0-9_]",  # range and negative range
+        "ab?",
+        "ab*",
+        "ab+",  # quantifiers
+        "ab{5}",
+        "ab{5,10}",
+        "ab{,10}",
+        "ab{5,}",  # repeaters
+        "ab|cd|ef",  # branch
+        "(foo)+",
+        "(['\"])[a-z]+\\1",
+        "(?:[a-z])(['\"])[a-z]+\\1",
+        "(?P<foo>['\"])[a-z]+(?P=foo)",  # groups
+        "^abc",  # beginning
+        "\\d",
+        "[\\d]",
+        "[^\\D]",
+        "\\w",
+        "[\\w]",
+        "[^\\W]",
+        "\\s",
+        "[\\s]",
+        "[^\\S]",  # categories
     ],
 )
 @pytest.mark.parametrize("encode", [False, True])
 def test_can_generate(pattern, encode):
     if encode:
         pattern = pattern.encode("ascii")
-    assert_all_examples(st.from_regex(pattern), re.compile(pattern).search)
+    with local_settings(settings(suppress_health_check=[HealthCheck.data_too_large])):
+        assert_all_examples(st.from_regex(pattern), re.compile(pattern).search)
 
 
 @pytest.mark.parametrize(
     "pattern",
     [
-        re.compile(u"\\Aa\\Z", re.IGNORECASE),
-        u"(?i)\\Aa\\Z",
-        re.compile(u"\\A[ab]\\Z", re.IGNORECASE),
-        u"(?i)\\A[ab]\\Z",
+        re.compile("\\Aa\\Z", re.IGNORECASE),
+        "(?i)\\Aa\\Z",
+        re.compile("\\A[ab]\\Z", re.IGNORECASE),
+        "(?i)\\A[ab]\\Z",
     ],
 )
 def test_literals_with_ignorecase(pattern):
     strategy = st.from_regex(pattern)
 
-    find_any(strategy, lambda s: s == u"a")
-    find_any(strategy, lambda s: s == u"A")
+    find_any(strategy, lambda s: s == "a")
+    find_any(strategy, lambda s: s == "A")
 
 
 @pytest.mark.parametrize(
-    "pattern", [re.compile(u"\\A[^a][^b]\\Z", re.IGNORECASE), u"(?i)\\A[^a][^b]\\Z"]
+    "pattern", [re.compile("\\A[^a][^b]\\Z", re.IGNORECASE), "(?i)\\A[^a][^b]\\Z"]
 )
 def test_not_literal_with_ignorecase(pattern):
     assert_all_examples(
         st.from_regex(pattern),
-        lambda s: s[0] not in (u"a", u"A") and s[1] not in (u"b", u"B"),
+        lambda s: s[0] not in ("a", "A") and s[1] not in ("b", "B"),
     )
 
 
 def test_any_doesnt_generate_newline():
-    assert_all_examples(st.from_regex(u"\\A.\\Z"), lambda s: s != u"\n")
+    assert_all_examples(st.from_regex("\\A.\\Z"), lambda s: s != "\n")
 
 
-@pytest.mark.parametrize("pattern", [re.compile(u"\\A.\\Z", re.DOTALL), u"(?s)\\A.\\Z"])
+@pytest.mark.parametrize("pattern", [re.compile("\\A.\\Z", re.DOTALL), "(?s)\\A.\\Z"])
 def test_any_with_dotall_generate_newline(pattern):
     find_any(
-        st.from_regex(pattern), lambda s: s == u"\n", settings(max_examples=10 ** 6)
+        st.from_regex(pattern), lambda s: s == "\n", settings(max_examples=10 ** 6)
     )
 
 
@@ -203,24 +193,14 @@ def test_any_with_dotall_generate_newline_binary(pattern):
 
 @pytest.mark.parametrize(
     "pattern",
-    [
-        u"\\d",
-        u"[\\d]",
-        u"[^\\D]",
-        u"\\w",
-        u"[\\w]",
-        u"[^\\W]",
-        u"\\s",
-        u"[\\s]",
-        u"[^\\S]",
-    ],
+    ["\\d", "[\\d]", "[^\\D]", "\\w", "[\\w]", "[^\\W]", "\\s", "[\\s]", "[^\\S]"],
 )
 @pytest.mark.parametrize("is_unicode", [False, True])
 @pytest.mark.parametrize("invert", [False, True])
 def test_groups(pattern, is_unicode, invert):
-    if u"d" in pattern.lower():
+    if "d" in pattern.lower():
         group_pred = is_digit
-    elif u"w" in pattern.lower():
+    elif "w" in pattern.lower():
         group_pred = is_word
     else:
         # Special behaviour due to \x1c, INFORMATION SEPARATOR FOUR
@@ -230,10 +210,10 @@ def test_groups(pattern, is_unicode, invert):
         pattern = pattern.swapcase()
         _p = group_pred
 
-        def group_pred(s):  # pylint:disable=function-redefined
+        def group_pred(s):
             return not _p(s)
 
-    pattern = u"^%s\\Z" % (pattern,)
+    pattern = "^%s\\Z" % (pattern,)
 
     compiler = unicode_regex if is_unicode else ascii_regex
     strategy = st.from_regex(compiler(pattern))
@@ -246,39 +226,38 @@ def test_groups(pattern, is_unicode, invert):
 
 
 def test_caret_in_the_middle_does_not_generate_anything():
-    r = re.compile(u"a^b")
+    r = re.compile("a^b")
 
     assert_no_examples(st.from_regex(r))
 
 
 def test_end_with_terminator_does_not_pad():
-    assert_all_examples(st.from_regex(u"abc\\Z"), lambda x: x[-3:] == u"abc")
+    assert_all_examples(st.from_regex("abc\\Z"), lambda x: x[-3:] == "abc")
 
 
 def test_end():
-    strategy = st.from_regex(u"\\Aabc$")
+    strategy = st.from_regex("\\Aabc$")
 
-    find_any(strategy, lambda s: s == u"abc")
-    find_any(strategy, lambda s: s == u"abc\n")
+    find_any(strategy, lambda s: s == "abc")
+    find_any(strategy, lambda s: s == "abc\n")
 
 
 def test_groupref_exists():
     assert_all_examples(
-        st.from_regex(u"^(<)?a(?(1)>)$"),
-        lambda s: s in (u"a", u"a\n", u"<a>", u"<a>\n"),
+        st.from_regex("^(<)?a(?(1)>)$"), lambda s: s in ("a", "a\n", "<a>", "<a>\n"),
     )
     assert_all_examples(
-        st.from_regex(u"^(a)?(?(1)b|c)$"), lambda s: s in (u"ab", u"ab\n", u"c", u"c\n")
+        st.from_regex("^(a)?(?(1)b|c)$"), lambda s: s in ("ab", "ab\n", "c", "c\n")
     )
 
 
 def test_impossible_negative_lookahead():
-    assert_no_examples(st.from_regex(u"(?!foo)foo"))
+    assert_no_examples(st.from_regex("(?!foo)foo"))
 
 
-@given(st.from_regex(u"(\\Afoo\\Z)"))
+@given(st.from_regex("(\\Afoo\\Z)"))
 def test_can_handle_boundaries_nested(s):
-    assert s == u"foo"
+    assert s == "foo"
 
 
 def test_groupref_not_shared_between_regex():
@@ -287,9 +266,13 @@ def test_groupref_not_shared_between_regex():
     st.tuples(st.from_regex("(a)\\1"), st.from_regex("(b)\\1")).example()
 
 
+@pytest.mark.skipif(
+    PYPY,  # Skip for now so we can test the rest
+    reason=r"Triggers bugs in poor handling of unicode in re for these implementations",
+)
 @given(st.data())
 def test_group_ref_is_not_shared_between_identical_regex(data):
-    pattern = re.compile(u"^(.+)\\1\\Z", re.UNICODE)
+    pattern = re.compile("^(.+)\\1\\Z", re.UNICODE)
     x = data.draw(base_regex_strategy(pattern))
     y = data.draw(base_regex_strategy(pattern))
     assume(x != y)
@@ -299,42 +282,42 @@ def test_group_ref_is_not_shared_between_identical_regex(data):
 
 @given(st.data())
 def test_does_not_leak_groups(data):
-    a = data.draw(base_regex_strategy(re.compile(u"^(a)\\Z")))
+    a = data.draw(base_regex_strategy(re.compile("^(a)\\Z")))
     assert a == "a"
-    b = data.draw(base_regex_strategy(re.compile(u"^(?(1)a|b)(.)\\Z")))
+    b = data.draw(base_regex_strategy(re.compile("^(?(1)a|b)(.)\\Z")))
     assert b[0] == "b"
 
 
 def test_positive_lookbehind():
-    find_any(st.from_regex(u".*(?<=ab)c"), lambda s: s.endswith(u"abc"))
+    find_any(st.from_regex(".*(?<=ab)c"), lambda s: s.endswith("abc"))
 
 
 def test_positive_lookahead():
-    st.from_regex(u"a(?=bc).*").filter(lambda s: s.startswith(u"abc")).example()
+    st.from_regex("a(?=bc).*").filter(lambda s: s.startswith("abc")).example()
 
 
 def test_negative_lookbehind():
     # no efficient support
-    strategy = st.from_regex(u"[abc]*(?<!abc)d")
+    strategy = st.from_regex("[abc]*(?<!abc)d")
 
-    assert_all_examples(strategy, lambda s: not s.endswith(u"abcd"))
-    assert_no_examples(strategy, lambda s: s.endswith(u"abcd"))
+    assert_all_examples(strategy, lambda s: not s.endswith("abcd"))
+    assert_no_examples(strategy, lambda s: s.endswith("abcd"))
 
 
 def test_negative_lookahead():
     # no efficient support
-    strategy = st.from_regex(u"^ab(?!cd)[abcd]*")
+    strategy = st.from_regex("^ab(?!cd)[abcd]*")
 
-    assert_all_examples(strategy, lambda s: not s.startswith(u"abcd"))
-    assert_no_examples(strategy, lambda s: s.startswith(u"abcd"))
+    assert_all_examples(strategy, lambda s: not s.startswith("abcd"))
+    assert_no_examples(strategy, lambda s: s.startswith("abcd"))
 
 
-@given(st.from_regex(u"^a+\\Z"))
+@given(st.from_regex("^a+\\Z"))
 def test_generates_only_the_provided_characters_given_boundaries(xs):
-    assert set(xs) == {u"a"}
+    assert set(xs) == {"a"}
 
 
-@given(st.from_regex(u"^(.)?\\1\\Z"))
+@given(st.from_regex("^(.)?\\1\\Z"))
 def test_group_backref_may_not_be_present(s):
     assert len(s) == 2
     assert s[0] == s[1]
@@ -342,15 +325,15 @@ def test_group_backref_may_not_be_present(s):
 
 @pytest.mark.skipif(sys.version_info[:2] < (3, 6), reason="requires Python 3.6")
 def test_subpattern_flags():
-    strategy = st.from_regex(u"(?i)\\Aa(?-i:b)\\Z")
+    strategy = st.from_regex("(?i)\\Aa(?-i:b)\\Z")
 
     # "a" is case insensitive
-    find_any(strategy, lambda s: s[0] == u"a")
-    find_any(strategy, lambda s: s[0] == u"A")
+    find_any(strategy, lambda s: s[0] == "a")
+    find_any(strategy, lambda s: s[0] == "A")
     # "b" is case sensitive
-    find_any(strategy, lambda s: s[1] == u"b")
+    find_any(strategy, lambda s: s[1] == "b")
 
-    assert_no_examples(strategy, lambda s: s[1] == u"B")
+    assert_no_examples(strategy, lambda s: s[1] == "B")
 
 
 def test_can_handle_binary_regex_which_is_not_ascii():
@@ -358,7 +341,7 @@ def test_can_handle_binary_regex_which_is_not_ascii():
     assert_all_examples(st.from_regex(bad), lambda x: bad in x)
 
 
-@pytest.mark.parametrize("pattern", [b".", u"."])
+@pytest.mark.parametrize("pattern", [b".", "."])
 def test_regex_have_same_type_as_pattern(pattern):
     @given(st.from_regex(pattern))
     def test_result_type(s):
@@ -368,44 +351,44 @@ def test_regex_have_same_type_as_pattern(pattern):
 
 
 def test_can_pad_strings_arbitrarily():
-    find_any(st.from_regex(u"a"), lambda x: x[0] != u"a")
-    find_any(st.from_regex(u"a"), lambda x: x[-1] != u"a")
+    find_any(st.from_regex("a"), lambda x: x[0] != "a")
+    find_any(st.from_regex("a"), lambda x: x[-1] != "a")
 
 
 def test_can_pad_empty_strings():
-    find_any(st.from_regex(u""), bool)
+    find_any(st.from_regex(""), bool)
     find_any(st.from_regex(b""), bool)
 
 
 def test_can_pad_strings_with_newlines():
-    find_any(st.from_regex(u"^$"), bool)
+    find_any(st.from_regex("^$"), bool)
     find_any(st.from_regex(b"^$"), bool)
 
 
 def test_given_multiline_regex_can_insert_after_dollar():
     find_any(
-        st.from_regex(re.compile(u"\\Ahi$", re.MULTILINE)),
-        lambda x: "\n" in x and x.split(u"\n")[1],
+        st.from_regex(re.compile("\\Ahi$", re.MULTILINE)),
+        lambda x: "\n" in x and x.split("\n")[1],
     )
 
 
 def test_given_multiline_regex_can_insert_before_caret():
     find_any(
-        st.from_regex(re.compile(u"^hi\\Z", re.MULTILINE)),
-        lambda x: "\n" in x and x.split(u"\n")[0],
+        st.from_regex(re.compile("^hi\\Z", re.MULTILINE)),
+        lambda x: "\n" in x and x.split("\n")[0],
     )
 
 
 def test_does_not_left_pad_beginning_of_string_marker():
-    assert_all_examples(st.from_regex(u"\\Afoo"), lambda x: x.startswith(u"foo"))
+    assert_all_examples(st.from_regex("\\Afoo"), lambda x: x.startswith("foo"))
 
 
 def test_bare_caret_can_produce():
-    find_any(st.from_regex(u"^"), bool)
+    find_any(st.from_regex("^"), bool)
 
 
 def test_bare_dollar_can_produce():
-    find_any(st.from_regex(u"$"), bool)
+    find_any(st.from_regex("$"), bool)
 
 
 def test_shared_union():
@@ -431,12 +414,12 @@ def test_issue_992_regression(data):
 @pytest.mark.parametrize(
     "pattern,matching_str",
     [
-        (u"a", u"a"),
-        (u"[Aa]", u"A"),
-        (u"[ab]*", u"abb"),
+        ("a", "a"),
+        ("[Aa]", "A"),
+        ("[ab]*", "abb"),
         (b"[Aa]", b"A"),
         (b"[ab]*", b"abb"),
-        (re.compile(u"[ab]*", re.IGNORECASE), u"aBb"),
+        (re.compile("[ab]*", re.IGNORECASE), "aBb"),
         (re.compile(b"[ab]", re.IGNORECASE), b"A"),
     ],
 )
@@ -451,15 +434,12 @@ def test_fullmatch_generates_example(pattern, matching_str):
 @pytest.mark.parametrize(
     "pattern,eqiv_pattern",
     [
-        (u"a", u"\\Aa\\Z"),
-        (u"[Aa]", u"\\A[Aa]\\Z"),
-        (u"[ab]*", u"\\A[ab]*\\Z"),
+        ("a", "\\Aa\\Z"),
+        ("[Aa]", "\\A[Aa]\\Z"),
+        ("[ab]*", "\\A[ab]*\\Z"),
         (b"[Aa]", br"\A[Aa]\Z"),
         (b"[ab]*", br"\A[ab]*\Z"),
-        (
-            re.compile(u"[ab]*", re.IGNORECASE),
-            re.compile(u"\\A[ab]*\\Z", re.IGNORECASE),
-        ),
+        (re.compile("[ab]*", re.IGNORECASE), re.compile("\\A[ab]*\\Z", re.IGNORECASE)),
         (re.compile(br"[ab]", re.IGNORECASE), re.compile(br"\A[ab]\Z", re.IGNORECASE)),
     ],
 )
