@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,17 +13,16 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import datetime
 import subprocess
 import sys
+from unittest import TestCase
 
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import example, given, unlimited
+from hypothesis import example, given, strategies as st
 from hypothesis._settings import (
+    Phase,
     Verbosity,
     default_variable,
     local_settings,
@@ -34,9 +31,9 @@ from hypothesis._settings import (
 )
 from hypothesis.database import ExampleDatabase
 from hypothesis.errors import InvalidArgument, InvalidState
-from hypothesis.stateful import GenericStateMachine, RuleBasedStateMachine, rule
+from hypothesis.stateful import RuleBasedStateMachine, rule
 from hypothesis.utils.conventions import not_set
-from tests.common.utils import checks_deprecated_behaviour, fails_with
+from tests.common.utils import counts_calls, fails_with
 
 
 def test_has_docstrings():
@@ -55,7 +52,7 @@ def setup_function(fn):
 def test_cannot_set_non_settings():
     s = settings()
     with pytest.raises(AttributeError):
-        s.databas_file = u"some_file"
+        s.databas_file = "some_file"
 
 
 def test_settings_uses_defaults():
@@ -92,7 +89,7 @@ def test_can_repeatedly_push_the_same_thing():
 
 def test_cannot_create_settings_with_invalid_options():
     with pytest.raises(InvalidArgument):
-        settings(a_setting_with_limited_options=u"spoon")
+        settings(a_setting_with_limited_options="spoon")
 
 
 def test_cannot_register_with_parent_and_settings_args():
@@ -188,11 +185,6 @@ def test_cannot_delete_a_setting():
         del x.foo
 
 
-@checks_deprecated_behaviour
-def test_setting_to_unlimited_is_not_error_yet():
-    settings(timeout=unlimited)
-
-
 def test_cannot_set_settings():
     x = settings()
     with pytest.raises(AttributeError):
@@ -227,25 +219,31 @@ def test_cannot_assign_default():
     assert settings().max_examples != 3
 
 
-def test_does_not_warn_if_quiet():
-    with pytest.warns(None) as rec:
-        note_deprecation("This is bad", since="RELEASEDAY", verbosity=Verbosity.quiet)
-    assert len(rec) == 0
-
-
 @settings(max_examples=7)
 @given(st.builds(lambda: settings.default))
 def test_settings_in_strategies_are_from_test_scope(s):
     assert s.max_examples == 7
 
 
-def test_settings_alone():
-    @settings()
-    def test_nothing():
-        pass
+TEST_SETTINGS_ALONE = """
+from hypothesis import settings
+from hypothesis.strategies import integers
 
-    with pytest.raises(InvalidArgument):
-        test_nothing()
+@settings()
+def test_settings_alone():
+    pass
+"""
+
+
+def test_settings_alone(testdir):
+    script = testdir.makepyfile(TEST_SETTINGS_ALONE)
+    result = testdir.runpytest(script)
+    out = "\n".join(result.stdout.lines)
+    assert (
+        "Using `@settings` on a test without `@given` is completely pointless." in out
+    )
+    assert "InvalidArgument" in out
+    assert result.ret == 1
 
 
 @fails_with(InvalidArgument)
@@ -312,6 +310,32 @@ def test_settings_apply_for_explicit_examples(x):
     assert settings.default.verbosity == Verbosity.verbose
 
 
+class TestGivenExampleSettingsExplicitCalled(TestCase):
+    """Real nasty edge case here.
+
+    in #2160, if ``example`` is after ``given`` but before ``settings``,
+    it will be completely ignored.
+
+    If we set phases to only ``explicit``, the test case will never be called!
+
+    We have to run an assertion outside of the test case itself.
+    """
+
+    @counts_calls
+    def call_target(self):
+        pass
+
+    @given(st.booleans())
+    @example(True)
+    @settings(phases=[Phase.explicit])
+    def test_example_explicit(self, x):
+        self.call_target()
+
+    def tearDown(self):
+        # In #2160, this is 0.
+        assert self.call_target.calls == 1
+
+
 def test_setattr_on_settings_singleton_is_error():
     # https://github.com/pandas-dev/pandas/pull/22679#issuecomment-420750921
     # Should be setting attributes on settings.default, not settings!
@@ -370,7 +394,7 @@ settings_step_count = 1
 @settings(stateful_step_count=settings_step_count)
 class StepCounter(RuleBasedStateMachine):
     def __init__(self):
-        super(StepCounter, self).__init__()
+        super().__init__()
         self.step_count = 0
 
     @rule()
@@ -404,8 +428,10 @@ def test_settings_decorator_applied_to_non_state_machine_class_raises_error():
 def test_assigning_to_settings_attribute_on_state_machine_raises_error():
     with pytest.raises(AttributeError):
 
-        class StateMachine(GenericStateMachine):
-            pass
+        class StateMachine(RuleBasedStateMachine):
+            @rule(x=st.none())
+            def a_rule(x):
+                assert x is None
 
         StateMachine.settings = settings()
 
@@ -426,11 +452,15 @@ def test_derandomise_with_explicit_database_is_invalid():
 @pytest.mark.parametrize(
     "kwargs",
     [
-        dict(max_examples=-1),
-        dict(buffer_size=-1),
-        dict(stateful_step_count=-1),
-        dict(deadline=-1),
-        dict(deadline=0),
+        {"max_examples": -1},
+        {"max_examples": 2.5},
+        {"buffer_size": -1},
+        {"stateful_step_count": -1},
+        {"stateful_step_count": 2.5},
+        {"deadline": -1},
+        {"deadline": 0},
+        {"deadline": True},
+        {"deadline": False},
     ],
 )
 def test_invalid_settings_are_errors(kwargs):
@@ -438,43 +468,22 @@ def test_invalid_settings_are_errors(kwargs):
         settings(**kwargs)
 
 
-@checks_deprecated_behaviour
-def test_boolean_deadlines():
-    settings(deadline=True)
-    with pytest.raises(InvalidArgument):
-        settings(deadline=False)
+def test_invalid_parent():
+    class NotSettings:
+        def __repr__(self):
+            return "(not settings repr)"
+
+    not_settings = NotSettings()
+
+    with pytest.raises(InvalidArgument) as excinfo:
+        settings(not_settings)
+
+    assert "parent=(not settings repr)" in str(excinfo.value)
 
 
-@checks_deprecated_behaviour
-def test_non_boolean_derandomize():
-    assert settings(derandomize=1).derandomize is True
-    assert settings(derandomize=0).derandomize is False
-
-
-@pytest.mark.parametrize("name", ["max_examples", "buffer_size", "stateful_step_count"])
-@checks_deprecated_behaviour
-def test_dubious_settings_deprecations(name):
-    settings(**{name: 2.5})
-    with pytest.raises(InvalidArgument):
-        settings(**{name: "2.5"})  # deprecation warning, then type cast error.
-
-
-@checks_deprecated_behaviour
-def test_buffer_size_deprecated():
-    assert settings(buffer_size=100).buffer_size == 100
-
-
-@checks_deprecated_behaviour
-def test_max_example_eq_0_warns_and_disables_generation():
-    # Terrible way to disable generation, but did predate the phases setting
-    # and existed in our test suite so it's not an error *just* yet.
-    @example(None)
-    @given(st.integers())
-    @settings(max_examples=0)
-    def inner(x):
-        calls[0] += 1
-        assert x is None
-
-    calls = [0]
-    inner()
-    assert calls[0] == 1
+def test_note_deprecation_checks_date():
+    with pytest.warns(None) as rec:
+        note_deprecation("This is bad", since="RELEASEDAY")
+    assert len(rec) == 1
+    with pytest.raises(AssertionError):
+        note_deprecation("This is way too old", since="1999-12-31")
