@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,22 +13,23 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import Phase, assume, given, settings
+from hypothesis import Phase, assume, given, settings, strategies as st
 from hypothesis.database import InMemoryExampleDatabase
 from hypothesis.errors import Flaky, MultipleFailures
 from hypothesis.internal.conjecture.engine import MIN_TEST_CALLS
-from tests.common.utils import capture_out, flaky, non_covering_examples
+from tests.common.utils import (
+    assert_output_contains_failure,
+    capture_out,
+    non_covering_examples,
+)
 
 
 def test_raises_multiple_failures_with_varying_type():
     target = [None]
 
-    @settings(database=None)
+    @settings(database=None, max_examples=100)
     @given(st.integers())
     def test(i):
         if abs(i) < 1000:
@@ -54,6 +53,7 @@ def test_raises_multiple_failures_with_varying_type():
 def test_raises_multiple_failures_when_position_varies():
     target = [None]
 
+    @settings(max_examples=100)
     @given(st.integers())
     def test(i):
         if abs(i) < 1000:
@@ -75,7 +75,7 @@ def test_raises_multiple_failures_when_position_varies():
 def test_replays_both_failing_values():
     target = [None]
 
-    @settings(database=InMemoryExampleDatabase())
+    @settings(database=InMemoryExampleDatabase(), max_examples=500)
     @given(st.integers())
     def test(i):
         if abs(i) < 1000:
@@ -97,7 +97,7 @@ def test_replays_slipped_examples_once_initial_bug_is_fixed(fix):
     target = []
     bug_fixed = False
 
-    @settings(database=InMemoryExampleDatabase())
+    @settings(database=InMemoryExampleDatabase(), max_examples=500)
     @given(st.integers())
     def test(i):
         if abs(i) < 1000:
@@ -130,7 +130,7 @@ def test_garbage_collects_the_secondary_key():
 
     db = InMemoryExampleDatabase()
 
-    @settings(database=db)
+    @settings(database=db, max_examples=500)
     @given(st.integers())
     def test(i):
         if bug_fixed:
@@ -188,8 +188,15 @@ def test_shrinks_both_failures():
         with pytest.raises(MultipleFailures):
             test()
 
-    assert "test(i=10000)" in o.getvalue()
-    assert "test(i=%d)" % (second_target[0],) in o.getvalue()
+    output = o.getvalue()
+
+    assert_output_contains_failure(
+        output, test, i=10000,
+    )
+
+    assert_output_contains_failure(
+        output, test, i=second_target[0],
+    )
 
 
 def test_handles_flaky_tests_where_only_one_is_flaky():
@@ -198,7 +205,7 @@ def test_handles_flaky_tests_where_only_one_is_flaky():
     target = []
     flaky_failed_once = [False]
 
-    @settings(database=InMemoryExampleDatabase())
+    @settings(database=InMemoryExampleDatabase(), max_examples=1000)
     @given(st.integers())
     def test(i):
         if abs(i) < 1000:
@@ -245,17 +252,26 @@ def test_can_disable_multiple_error_reporting(allow_multi):
     assert seen == {TypeError, ValueError}
 
 
-@flaky(max_runs=3, min_passes=2)
 def test_finds_multiple_failures_in_generation():
-    # Very rarely, this raises ZeroDivisionError instead of MultipleFailure,
-    # because we never generated NaN.  We therefore allow one additional run.
-    @settings(phases=[Phase.generate])
-    @given(st.lists(st.floats()))
+    special = []
+    seen = set()
+
+    @settings(phases=[Phase.generate], max_examples=100)
+    @given(st.integers(min_value=0))
     def test(x):
-        mean = sum(x) / len(x)  # ZeroDivisionError
-        if len(x) == 1:
-            assert mean == x[0]  # x[0] is nan
-        assert min(x) <= mean <= max(x)  # x contains nan or +&- inf
+        """Constructs a test so the 10th largeish example we've seen is a
+        special failure, and anything new we see after that point that
+        is larger than it is a different failure. This demonstrates that we
+        can keep generating larger examples and still find new bugs after that
+        point."""
+        if not special:
+            if len(seen) >= 10 and x <= 1000:
+                special.append(x)
+            else:
+                seen.add(x)
+        if special:
+            assert x in seen or (x <= special[0])
+        assert x not in special
 
     with pytest.raises(MultipleFailures):
         test()
