@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,28 +13,30 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import enum
+import sys
+from typing import Callable, Generic, List, Sequence, TypeVar, Union
 
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import given, infer
-from hypothesis._strategies import _strategies
+from hypothesis import given, infer, strategies as st
 from hypothesis.errors import (
     HypothesisDeprecationWarning,
     InvalidArgument,
     ResolutionFailed,
 )
-from hypothesis.internal.compat import PY2, integer_types
-from hypothesis.searchstrategy import types
-from hypothesis.searchstrategy.types import _global_type_lookup
-from tests.common.utils import checks_deprecated_behaviour
+from hypothesis.strategies._internal import types
+from hypothesis.strategies._internal.core import _strategies
+from hypothesis.strategies._internal.types import _global_type_lookup
+from tests.common.debug import assert_all_examples, find_any
+from tests.common.utils import checks_deprecated_behaviour, fails_with, temp_registered
 
 # Build a set of all types output by core strategies
 blacklist = [
     "builds",
+    "from_regex",
+    "from_type",
+    "ip_addresses",
     "iterables",
     "permutations",
     "random_module",
@@ -44,7 +44,7 @@ blacklist = [
     "runner",
     "sampled_from",
 ]
-types_with_core_strat = set(integer_types)
+types_with_core_strat = set()
 for thing in (
     getattr(st, name)
     for name in sorted(_strategies)
@@ -59,14 +59,17 @@ for thing in (
             continue
 
 
+def test_generic_sequence_of_integers_may_be_lists_or_bytes():
+    strat = st.from_type(Sequence[int])
+    find_any(strat, lambda x: isinstance(x, bytes))
+    find_any(strat, lambda x: isinstance(x, list))
+
+
 @pytest.mark.parametrize("typ", sorted(types_with_core_strat, key=str))
 def test_resolve_core_strategies(typ):
     @given(st.from_type(typ))
     def inner(ex):
-        if PY2 and issubclass(typ, integer_types):
-            assert isinstance(ex, integer_types)
-        else:
-            assert isinstance(ex, typ)
+        assert isinstance(ex, typ)
 
     inner()
 
@@ -91,27 +94,15 @@ def test_lookup_values_are_strategies():
 @pytest.mark.parametrize("typ", sorted(types_with_core_strat, key=str))
 def test_lookup_overrides_defaults(typ):
     sentinel = object()
-    try:
-        strat = types._global_type_lookup[typ]
-        st.register_type_strategy(typ, st.just(sentinel))
+    with temp_registered(typ, st.just(sentinel)):
         assert st.from_type(typ).example() is sentinel
-    finally:
-        st.register_type_strategy(typ, strat)
-        st.from_type.__clear_cache()
     assert st.from_type(typ).example() is not sentinel
 
 
-class ParentUnknownType(object):
+class ParentUnknownType:
     pass
 
 
-def test_can_resolve_trivial_types():
-    # Under Python 2, this inherits a special wrapper_descriptor slots
-    # thing from object.__init__, which chokes inspect.getargspec.
-    # from_type should and does work anyway; see issues #1655 and #1656.
-    st.from_type(ParentUnknownType).example()
-
-
 class UnknownType(ParentUnknownType):
     def __init__(self, arg):
         pass
@@ -125,51 +116,35 @@ def test_custom_type_resolution_fails_without_registering():
 
 def test_custom_type_resolution():
     sentinel = object()
-    try:
-        st.register_type_strategy(UnknownType, st.just(sentinel))
+    with temp_registered(UnknownType, st.just(sentinel)):
         assert st.from_type(UnknownType).example() is sentinel
         # Also covered by registration of child class
         assert st.from_type(ParentUnknownType).example() is sentinel
-    finally:
-        types._global_type_lookup.pop(UnknownType)
-        st.from_type.__clear_cache()
-        assert UnknownType not in types._global_type_lookup
 
 
 def test_custom_type_resolution_with_function():
     sentinel = object()
-    try:
-        st.register_type_strategy(UnknownType, lambda _: st.just(sentinel))
+    with temp_registered(UnknownType, lambda _: st.just(sentinel)):
         assert st.from_type(UnknownType).example() is sentinel
         assert st.from_type(ParentUnknownType).example() is sentinel
-    finally:
-        types._global_type_lookup.pop(UnknownType)
-        st.from_type.__clear_cache()
 
 
 def test_custom_type_resolution_with_function_non_strategy():
-    try:
-        st.register_type_strategy(UnknownType, lambda _: None)
+    with temp_registered(UnknownType, lambda _: None):
         with pytest.raises(ResolutionFailed):
             st.from_type(UnknownType).example()
         with pytest.raises(ResolutionFailed):
             st.from_type(ParentUnknownType).example()
-    finally:
-        types._global_type_lookup.pop(UnknownType)
 
 
 def test_errors_if_generic_resolves_empty():
-    try:
-        st.register_type_strategy(UnknownType, lambda _: st.nothing())
+    with temp_registered(UnknownType, lambda _: st.nothing()):
         fails_1 = st.from_type(UnknownType)
         with pytest.raises(ResolutionFailed):
             fails_1.example()
         fails_2 = st.from_type(ParentUnknownType)
         with pytest.raises(ResolutionFailed):
             fails_2.example()
-    finally:
-        types._global_type_lookup.pop(UnknownType)
-        st.from_type.__clear_cache()
 
 
 def test_cannot_register_empty():
@@ -189,7 +164,7 @@ def test_pulic_interface_works():
         fails.example()
 
 
-def test_given_can_infer_on_py2():
+def test_given_can_infer_from_manual_annotations():
     # Editing annotations before decorating is hilariously awkward, but works!
     def inner(a):
         pass
@@ -202,12 +177,13 @@ class EmptyEnum(enum.Enum):
     pass
 
 
-@checks_deprecated_behaviour
-def test_error_if_enum_is_empty():
-    assert st.from_type(EmptyEnum).is_empty
+@fails_with(InvalidArgument)
+@given(st.from_type(EmptyEnum))
+def test_error_if_enum_is_empty(x):
+    pass
 
 
-class BrokenClass(object):
+class BrokenClass:
     __init__ = "Hello!"
 
 
@@ -222,9 +198,100 @@ def test_uninspectable_from_type():
 
 
 @pytest.mark.parametrize(
-    "typ", sorted([x for x in _global_type_lookup if x.__module__ != "typing"], key=str)
+    "typ", sorted((x for x in _global_type_lookup if x.__module__ != "typing"), key=str)
 )
 @given(data=st.data())
 def test_can_generate_from_all_registered_types(data, typ):
     value = data.draw(st.from_type(typ), label="value")
     assert isinstance(value, typ)
+
+
+T = TypeVar("T")
+
+
+class MyGeneric(Generic[T]):
+    def __init__(self, arg: T) -> None:
+        self.arg = arg
+
+
+def using_generic(instance: MyGeneric[T]) -> T:
+    return instance.arg
+
+
+def using_concrete_generic(instance: MyGeneric[int]) -> int:
+    return instance.arg
+
+
+def test_generic_origin_empty():
+    with pytest.raises(ResolutionFailed):
+        find_any(st.builds(using_generic))
+
+
+_skip_callables_mark = pytest.mark.skipif(sys.version_info[:2] < (3, 7), reason="old")
+
+
+@pytest.mark.parametrize(
+    "generic",
+    (
+        Union[str, int],
+        Sequence[Sequence[int]],
+        MyGeneric[str],
+        # On Python <= 3.6, we always trigger the multi-registration guard clause
+        # and raise InvalidArgument on the first attempted registration.
+        pytest.param(Callable[..., str], marks=_skip_callables_mark),
+        pytest.param(Callable[[int], str], marks=_skip_callables_mark),
+    ),
+    ids=repr,
+)
+@pytest.mark.parametrize("strategy", [st.none(), lambda _: st.none()])
+@checks_deprecated_behaviour
+def test_generic_origin_with_type_args(generic, strategy):
+    try:
+        # Registering a generic type with args is deprecated
+        st.register_type_strategy(generic, strategy)
+        assert generic not in types._global_type_lookup
+        assert generic.__origin__ in types._global_type_lookup
+        # But trying to register another strategy does, since that could be
+        # a symptom of trying multiple registrations for different args
+        with pytest.raises(InvalidArgument):
+            st.register_type_strategy(generic, strategy)
+    finally:
+        st.from_type.__clear_cache()
+        for x in (generic, generic.__origin__):
+            types._global_type_lookup.pop(x, None)
+
+
+@pytest.mark.parametrize(
+    "generic",
+    (
+        Callable,
+        List,
+        Sequence,
+        # you can register types with all generic parameters
+        List[T],
+        Sequence[T],
+        # User-defined generics should also work
+        MyGeneric,
+        MyGeneric[T],
+    ),
+)
+def test_generic_origin_without_type_args(generic):
+    with temp_registered(generic, st.just("example")):
+        pass
+
+
+@pytest.mark.skipif(sys.version_info[:2] <= (3, 5), reason="typing on 3.5 is wild")
+def test_generic_origin_from_type():
+    with temp_registered(MyGeneric, st.builds(MyGeneric)):
+        find_any(st.from_type(MyGeneric[T]))
+        find_any(st.from_type(MyGeneric[int]))
+        find_any(st.from_type(MyGeneric))
+        find_any(st.builds(using_generic))
+        find_any(st.builds(using_concrete_generic))
+
+
+def test_generic_origin_concrete_builds():
+    with temp_registered(MyGeneric, st.builds(MyGeneric, st.integers())):
+        assert_all_examples(
+            st.builds(using_generic), lambda example: isinstance(example, int)
+        )
