@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,8 +13,6 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 from contextlib import contextmanager
 
 from hypothesis import find, given
@@ -24,7 +20,7 @@ from hypothesis._settings import Verbosity, settings
 from hypothesis.reporting import default as default_reporter, with_reporter
 from hypothesis.strategies import booleans, integers, lists
 from tests.common.debug import minimal
-from tests.common.utils import capture_out, checks_deprecated_behaviour, fails
+from tests.common.utils import capture_out, fails
 
 
 @contextmanager
@@ -50,7 +46,7 @@ def test_does_not_log_in_quiet_mode():
     with capture_verbosity() as o:
 
         @fails
-        @settings(verbosity=Verbosity.quiet)
+        @settings(verbosity=Verbosity.quiet, print_blob=False)
         @given(integers())
         def test_foo(x):
             assert False
@@ -68,11 +64,10 @@ def test_includes_progress_in_verbose_mode():
         )
     out = o.getvalue()
     assert out
-    assert u"Trying example: " in out
-    assert u"Falsifying example: " in out
+    assert "Trying example: " in out
+    assert "Falsifying example: " in out
 
 
-@checks_deprecated_behaviour
 def test_prints_initial_attempts_on_find():
 
     with capture_verbosity() as o:
@@ -86,23 +81,32 @@ def test_prints_initial_attempts_on_find():
                     return False
                 return x not in seen
 
-            find(integers(), not_first, settings=settings(verbosity=Verbosity.verbose))
+            find(
+                integers(),
+                not_first,
+                settings=settings(verbosity=Verbosity.verbose, max_examples=1000),
+            )
 
         foo()
 
-    assert u"Tried non-satisfying example" in o.getvalue()
+    assert "Trying example" in o.getvalue()
 
 
 def test_includes_intermediate_results_in_verbose_mode():
     with capture_verbosity() as o:
 
         @fails
-        @settings(verbosity=Verbosity.verbose, database=None, derandomize=True)
+        @settings(
+            verbosity=Verbosity.verbose,
+            database=None,
+            derandomize=True,
+            max_examples=100,
+        )
         @given(lists(integers(), min_size=1))
         def test_foo(x):
-            assert sum(x) < 1000000
+            assert sum(x) < 10000
 
         test_foo()
     lines = o.getvalue().splitlines()
-    assert len([l for l in lines if u"example" in l]) > 2
-    assert [l for l in lines if u"AssertionError" in l]
+    assert len([l for l in lines if "example" in l]) > 2
+    assert [l for l in lines if "AssertionError" in l]
