@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,8 +13,6 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import datetime as dt
 from uuid import UUID
 
@@ -25,23 +21,21 @@ from django.contrib.auth.models import User
 
 from hypothesis import HealthCheck, assume, given, infer, settings
 from hypothesis.control import reject
-from hypothesis.errors import HypothesisException, InvalidArgument
+from hypothesis.errors import (
+    HypothesisDeprecationWarning,
+    HypothesisException,
+    InvalidArgument,
+)
 from hypothesis.extra.django import (
     TestCase,
     TransactionTestCase,
     from_model,
     register_field_strategy,
 )
-from hypothesis.extra.django.models import (
-    add_default_field_mapping,
-    default_value,
-    models,
-)
-from hypothesis.internal.compat import text_type
 from hypothesis.internal.conjecture.data import ConjectureData
 from hypothesis.strategies import binary, just, lists
-from tests.common.utils import checks_deprecated_behaviour
 from tests.django.toystore.models import (
+    Car,
     Company,
     CompanyExtension,
     CouldBeCharming,
@@ -58,20 +52,10 @@ from tests.django.toystore.models import (
     Store,
 )
 
-register_field_strategy(CustomishField, just(u"a"))
+register_field_strategy(CustomishField, just("a"))
 
 
 class TestGetsBasicModels(TestCase):
-    @checks_deprecated_behaviour
-    def test_add_default_field_mapping_is_deprecated(self):
-        class UnregisteredCustomishField(CustomishField):
-            """Just to get deprecation warning when registered."""
-
-        add_default_field_mapping(UnregisteredCustomishField, just(u"a"))
-        with self.assertRaises(InvalidArgument):
-            # Double-registering is an error, and registry is shared.
-            register_field_strategy(UnregisteredCustomishField, just(u"a"))
-
     @given(from_model(Company))
     def test_is_company(self, company):
         self.assertIsInstance(company, Company)
@@ -121,61 +105,37 @@ class TestGetsBasicModels(TestCase):
 
     @given(from_model(Customish))
     def test_custom_field(self, x):
-        assert x.customish == u"a"
+        assert x.customish == "a"
 
     def test_mandatory_fields_are_mandatory(self):
         self.assertRaises(InvalidArgument, from_model(Store).example)
 
-    @checks_deprecated_behaviour
-    def test_mandatory_fields_are_mandatory_old(self):
-        self.assertRaises(InvalidArgument, models(Store).example)
-
     def test_mandatory_computed_fields_are_mandatory(self):
         with self.assertRaises(InvalidArgument):
             from_model(MandatoryComputed).example()
 
-    @checks_deprecated_behaviour
-    def test_mandatory_computed_fields_are_mandatory_old(self):
-        with self.assertRaises(InvalidArgument):
-            models(MandatoryComputed).example()
-
     def test_mandatory_computed_fields_may_not_be_provided(self):
         mc = from_model(MandatoryComputed, company=from_model(Company))
         self.assertRaises(RuntimeError, mc.example)
 
-    @checks_deprecated_behaviour
-    def test_mandatory_computed_fields_may_not_be_provided_old(self):
-        mc = models(MandatoryComputed, company=models(Company))
-        self.assertRaises(RuntimeError, mc.example)
-
-    @checks_deprecated_behaviour
-    @given(models(MandatoryComputed, company=default_value))
-    def test_mandatory_computed_field_default(self, x):
-        assert x.company.name == x.name + u"_company"
-
     @given(from_model(CustomishDefault, customish=infer))
     def test_customish_default_overridden_by_infer(self, x):
-        assert x.customish == u"a"
+        assert x.customish == "a"
 
     @given(from_model(CustomishDefault, customish=infer))
     def test_customish_infer_uses_registered_instead_of_default(self, x):
-        assert x.customish == u"a"
-
-    @checks_deprecated_behaviour
-    @given(models(CustomishDefault, customish=default_value))
-    def test_customish_default_generated(self, x):
-        assert x.customish == u"b"
+        assert x.customish == "a"
 
     @given(from_model(OddFields))
     def test_odd_fields(self, x):
         assert isinstance(x.uuid, UUID)
-        assert isinstance(x.slug, text_type)
-        assert u" " not in x.slug
-        assert isinstance(x.ipv4, text_type)
+        assert isinstance(x.slug, str)
+        assert " " not in x.slug
+        assert isinstance(x.ipv4, str)
         assert len(x.ipv4.split(".")) == 4
         assert all(int(i) in range(256) for i in x.ipv4.split("."))
-        assert isinstance(x.ipv6, text_type)
-        assert set(x.ipv6).issubset(set(u"0123456789abcdefABCDEF:."))
+        assert isinstance(x.ipv6, str)
+        assert set(x.ipv6).issubset(set("0123456789abcdefABCDEF:."))
 
     @given(from_model(ManyTimes))
     def test_time_fields(self, x):
@@ -186,7 +146,7 @@ class TestGetsBasicModels(TestCase):
     @given(from_model(Company))
     def test_no_null_in_charfield(self, x):
         # regression test for #1045.  Company just has a convenient CharField.
-        assert u"\x00" not in x.name
+        assert "\x00" not in x.name
 
     @given(binary(min_size=10))
     def test_foreign_key_primary(self, buf):
@@ -232,3 +192,16 @@ class TestValidatorInference(TestCase):
     @given(from_model(User))
     def test_user_issue_1112_regression(self, user):
         assert user.username
+
+
+class TestPosOnlyArg(TestCase):
+    @given(from_model(Car))
+    def test_user_issue_2369_regression(self, val):
+        pass
+
+    def test_from_model_argspec(self):
+        self.assertRaises(TypeError, from_model().example)
+        self.assertRaises(TypeError, from_model(Car, None).example)
+        self.assertWarns(
+            HypothesisDeprecationWarning, from_model(model=Customer).example
+        )
