@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,15 +13,11 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import find, given
-from hypothesis._strategies import defines_strategy
+from hypothesis import find, given, strategies as st
 from hypothesis.errors import InvalidArgument
-from tests.common.utils import checks_deprecated_behaviour
+from hypothesis.strategies._internal.core import defines_strategy
 
 
 def test_does_not_error_on_initial_calculation():
@@ -50,7 +44,6 @@ def test_errors_on_test_invocation():
         test()
 
 
-@checks_deprecated_behaviour
 def test_errors_on_find():
     s = st.lists(st.integers(), min_size=5, max_size=2)
     with pytest.raises(InvalidArgument):
