@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,11 +13,10 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
+from hypothesis import given, strategies as st
+from hypothesis.strategies._internal.types import _global_type_lookup
 
-import hypothesis.strategies as st
-from hypothesis import given
-from hypothesis.searchstrategy.types import _global_type_lookup
+TYPES = sorted((x for x in _global_type_lookup if x.__module__ != "typing"), key=str)
 
 
 def everything_except(excluded_types):
@@ -33,14 +30,7 @@ def everything_except(excluded_types):
 
 @given(
     excluded_types=st.lists(
-        st.sampled_from(
-            sorted(
-                [x for x in _global_type_lookup if x.__module__ != "typing"], key=str
-            )
-        ),
-        min_size=1,
-        max_size=3,
-        unique=True,
+        st.sampled_from(TYPES), min_size=1, max_size=3, unique=True,
     ).map(tuple),
     data=st.data(),
 )
