@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,8 +13,6 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import math
 import sys
 
@@ -25,7 +21,6 @@ import pytest
 from hypothesis import given
 from hypothesis.strategies import floats, integers, lists
 from tests.common.debug import minimal
-from tests.common.utils import checks_deprecated_behaviour
 
 
 def test_minimize_negative_int():
@@ -39,7 +34,7 @@ def test_positive_negative_int():
 
 
 boundaries = pytest.mark.parametrize(
-    u"boundary",
+    "boundary",
     sorted(
         [2 ** i for i in range(10)]
         + [2 ** i - 1 for i in range(10)]
@@ -84,7 +79,7 @@ def test_single_integer_range_is_range():
 
 
 def test_minimal_small_number_in_large_range():
-    assert minimal(integers((-2 ** 32), 2 ** 32), lambda x: x >= 101) == 101
+    assert minimal(integers((-(2 ** 32)), 2 ** 32), lambda x: x >= 101) == 101
 
 
 def test_minimal_small_sum_float_list():
@@ -114,7 +109,7 @@ def test_negative_floats_simplify_to_zero():
 
 
 def test_minimal_infinite_float_is_positive():
-    assert minimal(floats(), math.isinf) == float(u"inf")
+    assert minimal(floats(), math.isinf) == math.inf
 
 
 def test_can_minimal_infinite_negative_float():
@@ -131,7 +126,7 @@ def test_minimize_nan():
 
 def test_minimize_very_large_float():
     t = sys.float_info.max / 2
-    assert t <= minimal(floats(), lambda x: x >= t) < float(u"inf")
+    assert t <= minimal(floats(), lambda x: x >= t) < math.inf
 
 
 def is_integral(value):
@@ -143,8 +138,7 @@ def is_integral(value):
 
 def test_can_minimal_float_far_from_integral():
     minimal(
-        floats(),
-        lambda x: not (math.isnan(x) or math.isinf(x) or is_integral(x * (2 ** 32))),
+        floats(), lambda x: math.isfinite(x) and not is_integral(x * (2 ** 32)),
     )
 
 
@@ -172,7 +166,7 @@ def test_minimizes_lists_of_negative_ints_up_to_boundary():
 
 
 @pytest.mark.parametrize(
-    (u"left", u"right"),
+    ("left", "right"),
     [(0.0, 5e-324), (-5e-324, 0.0), (-5e-324, 5e-324), (5e-324, 1e-323)],
 )
 def test_floats_in_constrained_range(left, right):
@@ -214,25 +208,15 @@ def test_no_allow_infinity_lower(x):
     assert not math.isinf(x)
 
 
-class TestFloatsAreFloats(object):
+class TestFloatsAreFloats:
     @given(floats())
     def test_unbounded(self, arg):
         assert isinstance(arg, float)
 
-    @checks_deprecated_behaviour
-    @given(floats(min_value=0, max_value=2 ** 64 - 1))
-    def test_int_int(self, arg):
-        assert isinstance(arg, float)
-
     @given(floats(min_value=0, max_value=float(2 ** 64 - 1)))
     def test_int_float(self, arg):
         assert isinstance(arg, float)
 
-    @checks_deprecated_behaviour
-    @given(floats(min_value=float(0), max_value=2 ** 64 - 1))
-    def test_float_int(self, arg):
-        assert isinstance(arg, float)
-
     @given(floats(min_value=float(0), max_value=float(2 ** 64 - 1)))
     def test_float_float(self, arg):
         assert isinstance(arg, float)
