@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,8 +13,6 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import hashlib
 import math
 from random import Random
@@ -60,14 +56,14 @@ def clamp(lower, value, upper):
 
 class HypothesisSpec(RuleBasedStateMachine):
     def __init__(self):
-        super(HypothesisSpec, self).__init__()
+        super().__init__()
         self.database = None
 
-    strategies = Bundle(u"strategy")
-    strategy_tuples = Bundle(u"tuples")
-    objects = Bundle(u"objects")
-    basic_data = Bundle(u"basic")
-    varied_floats = Bundle(u"varied_floats")
+    strategies = Bundle("strategy")
+    strategy_tuples = Bundle("tuples")
+    objects = Bundle("objects")
+    basic_data = Bundle("basic")
+    varied_floats = Bundle("varied_floats")
 
     def teardown(self):
         self.clear_database()
@@ -116,7 +112,7 @@ class HypothesisSpec(RuleBasedStateMachine):
         def is_good(x):
             return bool(
                 Random(
-                    hashlib.md5((mixer + repr(x)).encode(u"utf-8")).digest()
+                    hashlib.sha384((mixer + repr(x)).encode("utf-8")).digest()
                 ).randint(0, level)
             )
 
@@ -140,9 +136,7 @@ class HypothesisSpec(RuleBasedStateMachine):
 
     @rule(target=strategies, left=varied_floats, right=varied_floats)
     def float_range(self, left, right):
-        for f in (math.isnan, math.isinf):
-            for x in (left, right):
-                assume(not f(x))
+        assume(math.isfinite(left) and math.isfinite(right))
         left, right = sorted((left, right))
         assert left <= right
         # exclude deprecated case where left = 0.0 and right = -0.0
@@ -162,7 +156,7 @@ class HypothesisSpec(RuleBasedStateMachine):
 
         def do_map(value):
             rep = repr(value)
-            random = Random(hashlib.md5((mixer + rep).encode(u"utf-8")).digest())
+            random = Random(hashlib.sha384((mixer + rep).encode("utf-8")).digest())
             if random.random() <= p:
                 return result1
             else:
@@ -193,10 +187,10 @@ class HypothesisSpec(RuleBasedStateMachine):
 
     @rule(strat=strategies)
     def repr_is_good(self, strat):
-        assert u" at 0x" not in repr(strat)
+        assert " at 0x" not in repr(strat)
 
 
-MAIN = __name__ == u"__main__"
+MAIN = __name__ == "__main__"
 
 TestHypothesis = HypothesisSpec.TestCase
 
