@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,10 +13,12 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
+from datetime import datetime
 
-import hypothesis.extra.pandas as pdst
-import hypothesis.strategies as st
+import pandas as pd
+
+from hypothesis import given, strategies as st
+from hypothesis.extra import pandas as pdst
 from tests.common.arguments import argument_validation_test, e
 
 BAD_ARGS = [
@@ -73,3 +73,13 @@ BAD_ARGS = [
 
 
 test_raise_invalid_argument = argument_validation_test(BAD_ARGS)
+
+lo, hi = pd.Timestamp(2017, 1, 1), pd.Timestamp(2084, 12, 21)
+
+
+@given(st.datetimes(min_value=lo, max_value=hi))
+def test_timestamp_as_datetime_bounds(dt):
+    # Would have caught https://github.com/HypothesisWorks/hypothesis/issues/2406
+    assert isinstance(dt, datetime)
+    assert lo <= dt <= hi
+    assert not isinstance(dt, pd.Timestamp)
