@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -25,19 +23,14 @@ drawing data from the distribution produced by some specifier is >=
 REQUIRED_P
 """
 
-
-from __future__ import absolute_import, division, print_function
-
 import collections
 import math
 import re
 
-import hypothesis.internal.reflection as reflection
-from hypothesis import settings as Settings
+from hypothesis import HealthCheck, settings as Settings
 from hypothesis.errors import UnsatisfiedAssumption
-from hypothesis.internal.conjecture.engine import (
-    ConjectureRunner as ConConjectureRunner,
-)
+from hypothesis.internal import reflection as reflection
+from hypothesis.internal.conjecture.engine import ConjectureRunner
 from hypothesis.strategies import (
     booleans,
     floats,
@@ -53,28 +46,31 @@ from hypothesis.strategies import (
 from tests.common.utils import no_shrink
 
 RUNS = 100
-REQUIRED_RUNS = 50
 
 
-INITIAL_LAMBDA = re.compile(u"^lambda[^:]*:\\s*")
+INITIAL_LAMBDA = re.compile("^lambda[^:]*:\\s*")
 
 
 def strip_lambda(s):
-    return INITIAL_LAMBDA.sub(u"", s)
+    return INITIAL_LAMBDA.sub("", s)
 
 
 class HypothesisFalsified(AssertionError):
     pass
 
 
-def define_test(specifier, predicate, condition=None):
+def define_test(
+    specifier, predicate, condition=None, p=0.5, suppress_health_check=(),
+):
+    required_runs = int(RUNS * p)
+
     def run_test():
         if condition is None:
 
             def _condition(x):
                 return True
 
-            condition_string = u""
+            condition_string = ""
         else:
             _condition = condition
             condition_string = strip_lambda(
@@ -93,27 +89,35 @@ def define_test(specifier, predicate, condition=None):
 
         successes = 0
         for _ in range(RUNS):
-            runner = ConConjectureRunner(
-                test_function, settings=Settings(max_examples=100, phases=no_shrink)
+            # We choose the max_examples a bit larger than default so that we
+            # run at least 100 examples outside of the small example generation
+            # part of the generation phase.
+            runner = ConjectureRunner(
+                test_function,
+                settings=Settings(
+                    max_examples=150,
+                    phases=no_shrink,
+                    suppress_health_check=suppress_health_check,
+                ),
             )
             runner.run()
             if runner.interesting_examples:
                 successes += 1
-                if successes >= REQUIRED_RUNS:
+                if successes >= required_runs:
                     return
         event = reflection.get_pretty_function_description(predicate)
         if condition is not None:
             event += "|"
             event += condition_string
 
-        description = (u"P(%s) ~ %d / %d = %.2f < %.2f") % (
+        description = ("P(%s) ~ %d / %d = %.2f < %.2f") % (
             event,
             successes,
             RUNS,
             successes / RUNS,
-            (REQUIRED_RUNS / RUNS),
+            (required_runs / RUNS),
         )
-        raise HypothesisFalsified(description + u" rejected")
+        raise HypothesisFalsified(description + " rejected")
 
     return run_test
 
@@ -127,32 +131,30 @@ test_can_produce_large_negative_integers = define_test(integers(), lambda x: x <
 
 
 def long_list(xs):
-    return len(xs) >= 20
+    return len(xs) >= 10
 
 
 test_can_produce_unstripped_strings = define_test(text(), lambda x: x != x.strip())
 
 test_can_produce_stripped_strings = define_test(text(), lambda x: x == x.strip())
 
-test_can_produce_multi_line_strings = define_test(text(), lambda x: u"\n" in x)
+test_can_produce_multi_line_strings = define_test(text(), lambda x: "\n" in x)
 
 test_can_produce_ascii_strings = define_test(
     text(), lambda x: all(ord(c) <= 127 for c in x)
 )
 
 test_can_produce_long_strings_with_no_ascii = define_test(
-    text(min_size=5), lambda x: all(ord(c) > 127 for c in x)
+    text(min_size=5), lambda x: all(ord(c) > 127 for c in x), p=0.1
 )
 
 test_can_produce_short_strings_with_some_non_ascii = define_test(
     text(), lambda x: any(ord(c) > 127 for c in x), condition=lambda x: len(x) <= 3
 )
 
-test_can_produce_positive_infinity = define_test(floats(), lambda x: x == float(u"inf"))
+test_can_produce_positive_infinity = define_test(floats(), lambda x: x == math.inf)
 
-test_can_produce_negative_infinity = define_test(
-    floats(), lambda x: x == float(u"-inf")
-)
+test_can_produce_negative_infinity = define_test(floats(), lambda x: x == -math.inf)
 
 test_can_produce_nan = define_test(floats(), math.isnan)
 
@@ -162,7 +164,7 @@ test_can_produce_floats_near_right = define_test(floats(0, 1), lambda t: t > 0.8
 
 test_can_produce_floats_in_middle = define_test(floats(0, 1), lambda t: 0.2 <= t <= 0.8)
 
-test_can_produce_long_lists = define_test(lists(integers()), long_list)
+test_can_produce_long_lists = define_test(lists(integers()), long_list, p=0.3)
 
 test_can_produce_short_lists = define_test(lists(integers()), lambda x: len(x) <= 10)
 
@@ -212,6 +214,7 @@ test_mixing_is_sometimes_distorted = define_test(
     lists(booleans() | tuples()),
     distorted,
     condition=lambda x: len(set(map(type, x))) == 2,
+    suppress_health_check=[HealthCheck.filter_too_much],
 )
 
 test_mixes_2_reasonably_often = define_test(
@@ -219,7 +222,7 @@ test_mixes_2_reasonably_often = define_test(
 )
 
 test_partial_mixes_3_reasonably_often = define_test(
-    lists(booleans() | tuples() | just(u"hi")),
+    lists(booleans() | tuples() | just("hi")),
     lambda x: 1 < len(set(map(type, x))) < 3,
     condition=bool,
 )
@@ -360,3 +363,8 @@ for i in range(4):
     )"""
         % (i, i)
     )
+
+
+test_long_duplicates_strings = define_test(
+    tuples(text(), text()), lambda s: len(s[0]) >= 5 and s[0] == s[1],
+)
