@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,21 +13,19 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import os
+import re
 import shlex
 import subprocess
 import sys
 from datetime import datetime
 from glob import glob
 
+from coverage.config import CoverageConfig
+
 import hypothesistooling as tools
-import hypothesistooling.installers as install
-import hypothesistooling.projects.conjecturerust as cr
-import hypothesistooling.projects.hypothesispython as hp
-import hypothesistooling.projects.hypothesisruby as hr
-import hypothesistooling.releasemanagement as rm
+from hypothesistooling import installers as install, releasemanagement as rm
+from hypothesistooling.projects import conjecturerust as cr, hypothesispython as hp
 from hypothesistooling.scripts import pip_tool
 
 TASKS = {}
@@ -80,16 +76,6 @@ def lint():
         "--config",
         os.path.join(tools.ROOT, ".flake8"),
     )
-    # Check for redefined test functions, where e.g. a copy-pasted definition
-    # will shadow the earlier test and Pytest never sees or executes it.
-    pip_tool(
-        "pylint",
-        "--score=n",
-        "--jobs=0",
-        "--disable=all",
-        "--enable=function-redefined",
-        "hypothesis-python/tests/",
-    )
 
 
 HEAD = tools.hash_for_name("HEAD")
@@ -152,8 +138,6 @@ CURRENT_YEAR = datetime.utcnow().year
 
 
 HEADER = """
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
@@ -175,10 +159,11 @@ HEADER = """
 @task()
 def format():
     def should_format_file(path):
-        if "vendor" in path.split(os.path.sep):
-            return False
         return path.endswith(".py")
 
+    def should_format_doc_file(path):
+        return path.endswith((".rst", ".md"))
+
     changed = tools.modified_files()
 
     format_all = os.environ.get("FORMAT_ALL", "").lower() == "true"
@@ -189,11 +174,21 @@ def format():
 
     files = tools.all_files() if format_all else changed
 
+    doc_files_to_format = [f for f in sorted(files) if should_format_doc_file(f)]
+    pip_tool("blacken-docs", *doc_files_to_format)
+
     files_to_format = [f for f in sorted(files) if should_format_file(f)]
 
     if not files_to_format:
         return
 
+    # .coveragerc lists several regex patterns to treat as nocover pragmas, and
+    # we want to find (and delete) cases where # pragma: no cover is redundant.
+    config = CoverageConfig()
+    config.from_file(os.path.join(hp.BASE_DIR, ".coveragerc"), our_file=True)
+    pattern = "|".join(l for l in config.exclude_list if "pragma" not in l)
+    unused_pragma_pattern = re.compile(f"({pattern}).*# pragma: no cover")
+
     for f in files_to_format:
         lines = []
         with open(f, encoding="utf-8") as o:
@@ -209,6 +204,8 @@ def format():
                 if "END HEADER" in l and not header_done:
                     lines = []
                     header_done = True
+                elif unused_pragma_pattern.search(l) is not None:
+                    lines.append(l.replace("# pragma: no cover", ""))
                 else:
                     lines.append(l)
         source = "".join(lines).strip()
@@ -232,12 +229,12 @@ def format():
         "--remove-unused-variables",
         *files_to_format,
     )
-    pip_tool("pyupgrade", "--keep-percent-format", *files_to_format)
+    pip_tool("pyupgrade", "--keep-percent-format", "--py3-plus", *files_to_format)
     pip_tool("isort", *files_to_format)
-    pip_tool("black", *files_to_format)
+    pip_tool("black", "--target-version=py35", *files_to_format)
 
 
-VALID_STARTS = ("# coding=utf-8", "#!/usr/bin/env python")
+VALID_STARTS = (HEADER.split()[0], "#!/usr/bin/env python")
 
 
 @task()
@@ -248,7 +245,7 @@ def check_format():
     for f in tools.all_files():
         if not f.endswith(".py"):
             continue
-        with open(f, "r", encoding="utf-8") as i:
+        with open(f, encoding="utf-8") as i:
             start = i.read(n)
             if not any(start.startswith(s) for s in VALID_STARTS):
                 print("%s has incorrect start %r" % (f, start), file=sys.stderr)
@@ -271,7 +268,13 @@ def compile_requirements(upgrade=False):
     for f in glob(os.path.join("requirements", "*.in")):
         base, _ = os.path.splitext(f)
         pip_tool(
-            "pip-compile", *extra, f, "--output-file", base + ".txt", cwd=tools.ROOT
+            "pip-compile",
+            *extra,
+            f,
+            "hypothesis-python/setup.py",
+            "--output-file",
+            base + ".txt",
+            cwd=tools.ROOT,
         )
 
 
@@ -372,24 +375,26 @@ def run_tox(task, version):
     pip_tool("tox", "-e", task, env=env, cwd=hp.HYPOTHESIS_PYTHON)
 
 
-PY27 = "2.7.14"
-PY35 = "3.5.5"
-PY36 = "3.6.8"
-PY37 = "3.7.0"
-PYPY2 = "pypy2.7-5.10.0"
-PYPY3 = "pypy3.5-5.10.1"
+# Via https://github.com/pyenv/pyenv/tree/master/plugins/python-build/share/python-build
+PY35 = "3.5.7"
+PY36 = "3.6.9"
+PY37 = "3.7.4"
+PY38 = "3.8.0"
+PY39 = "3.9-dev"
+PYPY35 = "pypy3.5-7.0.0"
+PYPY36 = "pypy3.6-7.1.1"
 
 
 @task()
 def install_core():
-    install.python_executable(PY27)
     install.python_executable(PY36)
 
 
-ALIASES = {PYPY2: "pypy", PYPY3: "pypy3"}
+# ALIASES are the executable names for each Python version
+ALIASES = {PYPY35: "pypy3", PYPY36: "pypy3"}
 
-for n in [PY27, PY35, PY36, PY37]:
-    major, minor, patch = n.split(".")
+for n in [PY35, PY36, PY37, PY38, PY39]:
+    major, minor, patch = n.replace("-dev", ".").split(".")
     ALIASES[n] = "python%s.%s" % (major, minor)
 
 
@@ -402,11 +407,6 @@ python_tests = task(
 )
 
 
-@python_tests
-def check_py27():
-    run_tox("py27-full", PY27)
-
-
 @python_tests
 def check_py35():
     run_tox("py35-full", PY35)
@@ -423,23 +423,23 @@ def check_py37():
 
 
 @python_tests
-def check_pypy():
-    run_tox("pypy-full", PYPY2)
+def check_py38():
+    run_tox("py38-full", PY38)
 
 
 @python_tests
-def check_pypy3():
-    run_tox("pypy3-full", PYPY3)
+def check_py39():
+    run_tox("py39-full", PY39)
 
 
 @python_tests
-def check_py27_typing():
-    run_tox("py27typing", PY27)
+def check_pypy35():
+    run_tox("pypy3-full", PYPY35)
 
 
 @python_tests
-def check_pypy_with_tracer():
-    run_tox("pypy-with-tracer", PYPY2)
+def check_pypy36():
+    run_tox("pypy3-full", PYPY36)
 
 
 def standard_tox_task(name):
@@ -447,14 +447,15 @@ def standard_tox_task(name):
 
 
 standard_tox_task("nose")
-standard_tox_task("pytest30")
+standard_tox_task("pytest43")
 
-for n in [20, 21, 22, 111]:
+for n in [22, 30]:
     standard_tox_task("django%d" % (n,))
-for n in [19, 20, 21, 22, 23, 24, 25]:
+for n in [19, 20, 21, 22, 23, 24, 25, 100]:
     standard_tox_task("pandas%d" % (n,))
 
 standard_tox_task("coverage")
+standard_tox_task("conjecture-coverage")
 
 
 @task()
@@ -467,21 +468,11 @@ examples_task = task(
 )
 
 
-@examples_task
-def check_examples2():
-    run_tox("examples2", PY27)
-
-
 @examples_task
 def check_examples3():
     run_tox("examples3", PY36)
 
 
-@python_tests
-def check_unicode():
-    run_tox("unicode", PY27)
-
-
 @task()
 def check_whole_repo_tests():
     install.ensure_shellcheck()
@@ -495,30 +486,11 @@ def shell():
     IPython.start_ipython([])
 
 
-def ruby_task(fn):
-    return task(if_changed=(hr.HYPOTHESIS_RUBY,))(fn)
-
-
-@ruby_task
-def lint_ruby():
-    hr.rake_task("checkformat")
-
-
-@ruby_task
-def check_ruby_tests():
-    hr.rake_task("test")
-
-
 @task()
 def python(*args):
     os.execv(sys.executable, (sys.executable,) + args)
 
 
-@task()
-def bundle(*args):
-    hr.bundle(*args)
-
-
 rust_task = task(if_changed=(cr.BASE_DIR,))
 
 
